-------------------------------------------------------------
-- MSS copyright 1999-2007
--	Filename:  AGC10B.VHD
-- Author: Alain Zarembowitch / MSS
--	Version: 5
--	Date last modified: 5-11-07
-- Inheritance: 	AGC10 rev4 10-24-05
--
-- Description:  Part of the front-end AGC. Aims at preventing saturation at 
-- the external A/D converters. The actual gain control is implemented in the
-- analog front-end circuit. 
-- KEY ASSUMPTION: GAIN CONTROL IS NOT INVERTED. 0 FOR MINIMUM GAIN, ALL 1'S FOR MAXIMUM GAIN
-- 
-- Algorithm: Measures the input signal magnitude. Adjusts the front-end gain to make full
-- use of the A/D converters dynamic range. The AGC loop bandwidth is user selectable. 
-- The magnitude is averaged over 1024 input samples. The gain control is updated once 
-- every DECIMATION input samples. 
-- The algorithm is based on the magnitude of the complex (I,Q) input samples. 
-- Size summary (Spartan-3 XC3S400), including embedded polar.vhd for magnitude computation: 
--   FF: 149, LUT4s: 291, Slices: 194, Block RAMs: 2
--   Speed: 102 MHz	(speed grade -4)
--
-- Rev 3. 8-25-05 AZ
-- Replaced POLAR.VHD by higher-precision/smaller POLAR2.VHD
--
-- Rev 4. 10-26-05 AZ
-- Replaced POLAR.VHD by higher-precision POLAR3.VHD
--
-- Rev 5 5-11-07 AZ
-- Added a 'severe' saturation detection and avoidance mechanism
-- to prevent the magnitude-based AGC from reaching a false stability point.
-- Tested (Slacker.com)
---------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity AGC10B is
    port ( 

	--// GLOBAL CLOCKS
   CLK : in std_logic;				-- System clock for this FPGA

	--// INPUT DATA STREAM
	DATA_I_IN: in std_logic_vector(11 downto 0);
	DATA_Q_IN: in std_logic_vector(11 downto 0);
		-- input samples, 12-bit precision
		-- read at rising edge of CLK when SAMPLE_CLK_IN = '1'.
		-- Format: 2's complement.
	SAMPLE_CLK_IN: in std_logic;
		-- sample clock. 

	--// CONTROLS 
	DECIMATION: in std_logic_vector(23 downto 0);
		-- used to control the AGC loop gain and prevent instabilities.
		-- Adjust as a function of the gain control sensitivity or bandwidth.
		-- minimum 1. 0 is an illegal value
		-- The gain control value AGC_DAC is updated once every DECIMATION input samples. 
   TARGET_MAGNITUDE: in std_logic_vector(13 downto 0);
		-- Use this target to control the nominal A/D converter range used in the average.
		-- May be helpful in cases where the peak to average ratio is high, or the
		-- signal duty cycle is small, or... etc.

 	--// AGC OUTPUTS
	AGC_DAC: out std_logic_vector(9 downto 0);
		-- gain control. May need to be inverted depending on the analog front-end.
		-- Here, maximum gain really means maximum gain.
	AGC_DAC_SAMPLE_CLK: out std_logic;
		-- read AGC_DAC at the rising edge of CLK when AGC_DAC_SAMPLE_CLK = 1.
	
	--// TEST POINTS
	S1: out std_logic_vector(7 downto 0);
	S1_CLK: out std_logic;
	S2: out std_logic_vector(7 downto 0);
	S2_CLK: out std_logic


			);
end entity;

architecture behavioral of AGC10B is
--------------------------------------------------------
--      COMPONENTS
--------------------------------------------------------
component POLAR3 is port (  
		--// CLOCK, RESET
		CLK: in std_logic;
			-- Reference clock, synchronous design.

		--// INPUTS 
		X_IN: in std_logic_vector(11 downto 0);
		Y_IN: in std_logic_vector(11 downto 0);
			-- real and imaginary input sample
			-- 2's complement
		SAMPLE_CLK_IN: in std_logic;
			-- read input complex sample at rising_edge of CLK when SAMPLE_CLK_IN = '1'

		--// OUTPUTS
		PHASE_OUT: out std_logic_vector(10 downto 0) :="00000000000";
			-- phase expressed as 0 - 360 deg unsigned, 0.175 deg steps
		MAGNITUDE_OUT: out std_logic_vector(13 downto 0) := "00000000000000";
			-- 4 x the true magnitude in unsigned representation (if you want
			-- signed simply add a zero before the MSb because a magnitude
			-- is always positive). To get the true scale the divide this
			-- number by four (or simply chop off the two least significant
			-- bits). 
		SAMPLE_CLK_OUT: out std_logic
			-- read output at rising_edge of CLK when SAMPLE_CLK_OUT = '1'
			-- Latency is 5 CLKs.
 );
end component;  
--------------------------------------------------------
--     SIGNALS
--------------------------------------------------------
-- Suffix _D indicates a one CLK delayed version of the net with the same name
-- Suffix _E indicates an extended precision version of the net with the same name
-- Suffix _N indicates an inverted version of the net with the same name
signal ZERO: std_logic;
signal MAGNITUDE: std_logic_vector(13 downto 0);
signal MAGNITUDE_E: std_logic_vector(23 downto 0);
signal MAGNITUDE_SAMPLE_CLK: std_logic;
signal MAGNITUDE_SAMPLE_CLK_D: std_logic;
signal DECIMATION_COUNTER: std_logic_vector(23 downto 0);
signal DECIMATION_COUNTER_INC: std_logic_vector(23 downto 0);
signal DECIMATED_SAMPLE_CLK: std_logic;
signal MAGNITUDE_ACC: std_logic_vector(23 downto 0);
signal MAGNITUDE_ACC_DIV1024: std_logic_vector(23 downto 0);
signal GAIN_LOCAL: std_logic_vector(9 downto 0):= "0000000000";
signal SATURATION_AVRG: std_logic_vector(15 downto 0);
signal SATURATION: std_logic;		

--------------------------------------------------------
--      IMPLEMENTATION
--------------------------------------------------------

begin
ZERO <= '0';

-- convert input signals to magnitude. 
-- Latency is 5 CLKs.
POLAR_001: POLAR3 port map(
	CLK => CLK,
	X_IN => DATA_I_IN, 
	Y_IN => DATA_Q_IN, 
	SAMPLE_CLK_IN => SAMPLE_CLK_IN,
	PHASE_OUT => open, 	-- phase is not used here, only magnitude.
	MAGNITUDE_OUT => MAGNITUDE,
	SAMPLE_CLK_OUT => MAGNITUDE_SAMPLE_CLK	-- same frequency as SAMPLE_CLK_IN.
	);


-- average magnitude	(or peak magnitude) over 1024 samples using a leaky integrator.
MAGNITUDE_E <= "0000000000" & MAGNITUDE;
MAGNITUDE_ACC_DIV1024 <= "0000000000" & MAGNITUDE_ACC(23 downto 10);

AVERAGE_GEN: process(CLK)
begin
	if rising_edge(CLK) then
		MAGNITUDE_SAMPLE_CLK_D <= MAGNITUDE_SAMPLE_CLK;

		if(MAGNITUDE_SAMPLE_CLK = '1') then
			MAGNITUDE_ACC <= MAGNITUDE_ACC +  MAGNITUDE_E - MAGNITUDE_ACC_DIV1024;
		end if;
	end if;
end process;					


-- Decimation counter
-- The main purpose is to control the AGC loop gain. 
DECIMATION_COUNTER_INC <= DECIMATION_COUNTER + 1;

DECIMATION_COUNTER_GEN: process(CLK)
begin
	if rising_edge(CLK) then
		if(MAGNITUDE_SAMPLE_CLK_D = '1') then
			if(DECIMATION_COUNTER_INC = DECIMATION) then
				DECIMATION_COUNTER <= (others => '0');
			else
				DECIMATION_COUNTER <= DECIMATION_COUNTER_INC;
			end if;
		end if;
	end if;
end process;

DECIMATED_SAMPLE_CLK <= MAGNITUDE_SAMPLE_CLK_D when (DECIMATION_COUNTER_INC = DECIMATION) else '0';


-- compare average magnitude with target
-- Adjust gain.
-- KEY ASSUMPTION: 0 FOR MAXIMUM GAIN, ALL 1'S FOR MINIMUM GAIN
GAIN_GEN: process(CLK)
begin
	if rising_edge(CLK) then
		AGC_DAC_SAMPLE_CLK <= DECIMATED_SAMPLE_CLK;

		if(DECIMATED_SAMPLE_CLK = '1') then
			if(SATURATION_AVRG(15 downto 10) > 10) then
				-- special case. non-linear. several saturation (more than 10/32 = 30% of the time)
				-- signal is too strong. reduce gain
				if(GAIN_LOCAL /= "1111111111") then
					GAIN_LOCAL <= GAIN_LOCAL + 1;
				end if;
			elsif(MAGNITUDE_ACC(23 downto 10) < TARGET_MAGNITUDE) then
				-- increase gain
				if(GAIN_LOCAL /= 0) then
					GAIN_LOCAL <= GAIN_LOCAL - 1;
				end if;
			else
				-- signal is too strong. reduce gain
				if(GAIN_LOCAL /= "1111111111") then
					GAIN_LOCAL <= GAIN_LOCAL + 1;
				end if;
			end if;
		end if;
	end if;
end process;			


AGC_DAC <= GAIN_LOCAL;

-- detect severe saturation
SATURATION <= '1' when (DATA_I_IN(11 downto 5) = "0111111") or 
		  (DATA_I_IN(11 downto 5) = "1000000") or 
		  (DATA_Q_IN(11 downto 5) = "0111111") or 
		  (DATA_Q_IN(11 downto 5) = "1000000") else '0';
		  
		  
SATURATION_DETECTION_001: process(CLK)
begin
	if rising_edge(CLK) then
		if(SAMPLE_CLK_IN = '1') then
			-- running average over 1024 consecutive samples.
			-- series converges towards 1024 * (32 * average saturation)
			-- SATURATION_AVRG(15 downto 10) = 32 when saturating 100% of the time, or 0 when no saturation, or anywhere inbetween
			SATURATION_AVRG <= SATURATION_AVRG + ("0000000000" & SATURATION & "00000") - ("0000000000" & SATURATION_AVRG(15 downto 10));
		end if;
	end if;
end process;

--// Test points
S1 <= MAGNITUDE(13 downto 6);
S1_CLK <= MAGNITUDE_SAMPLE_CLK;
S2 <= MAGNITUDE_ACC(23 downto 16);
S2_CLK <= DECIMATED_SAMPLE_CLK;


end behavioral;
