-------------------------------------------------------------
-- Filename:  COM1400_RX_INTERFACE.VHD
-- Authors: 
--		Alain Zarembowitch / MSS
--		Adam Kwiatkowski / MSS
-- Version: Rev 1
-- Last modified: 2/26/08
-- Inheritance: 	COM1435A_013
--
-- description:  
-- Code template for COM-1400 FPGA development platform
-- The main objective is to give another example of VHDL code development.
-- Another objective is to use the code to test various COM-300X receivers
-- and display the resulting input signals via ComScope.
-- This code is compatible with all COM-300X interfaces, including the 
-- COM-3008 high-speed DDR interface at 210 MSamples/s.
-- Includes USB 2.0 driver 
--
-- There is only one control register:
-- REG0: AGC time-constant, expressed in number of rising-edge input samples.
-- 0 -> AGC is updated every input sample
-- 1 -> every 2 input samples
-- 2 -> every 4 input samples.....
-- n -> every 2^n input samples.
--
-- ComScope displays the following signals:
-- Signal1 Trace1: positive edge I input signal
-- Signal1 Trace2: negative edge I input signal
-- Signal1 Trace3: positive edge Q input signal
-- Signal1 Trace4: negative edge Q input signal
-- Signal2 Trace2: AGC gain control

---------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;
Library UNISIM;
use UNISIM.vcomponents.all;

entity COM1400_RX_INTERFACE is
    Port ( 
		--GLOBAL CLOCKS
	   USB_CLK60 : in std_logic;	
			-- 60 MHz reference clock. Generated by the USB PHY on this board.

		--// Left connector, 40 pin female 2mm connector. 
		-- Pin 1 is reserved for input clock (90 MHz, DDR clock)
		-- Pins 10,20,30,40 reserved for Ground
		-- Pins 37,38,39 reserved for JTAG
		-- Pins 35-36 reserved for ComBlock Monitoring and Control.
		-- pullup pin B13 for interface with DACs.
		LEFT_CONNECTOR: inout std_logic_vector(34 downto 1);

		--// Right/Bottom connectors: 40 pin male 2mm connector. 
		-- Pin 1 is reserved to forward clock to following module.
		-- Pins 10,20,30,40 reserved for Ground
		-- Pins 37,38,39 reserved for JTAG
		RIGHT_CONNECTOR: inout std_logic_vector(34 downto 1);
		BOTTOM_CONNECTOR: inout std_logic_vector(34 downto 1);

		--// USB PHY interface (SMSC GT3200 IC)
		-- Direct connection between the USB20 component and the USB PHY.
		-- Synchronous with USB_CLK60G clock
		USB_RESET: out std_logic;
--		USB_DATABUS16_8: out std_logic;
--		USB_SUSPENDN: out std_logic;
		USB_XCVRSELECT: out std_logic;
		USB_TERMSELECT: out std_logic;
		USB_OPMODE: out std_logic_vector(1 downto 0);
			-- operational mode
			-- 00 = normal operation
			-- 01 = non-driving (all terminations removed)
			-- 10 = disable bit stuffing and NRZI encoding (unused)
			-- 11 = reserved (unused)
		USB_LINESTATE: in std_logic_vector(1 downto 0);	
			--* possible race condition with USB_CLK60G. Reclock at falling edge.

		USB_TXVALID: out std_logic;
		USB_TXREADY: in std_logic;	
			--* possible race condition with USB_CLK60G. Reclock at falling edge.
--		USB_VALIDH: inout std_logic;	-- VALIDH is not used in 8-bit mode
		USB_RXVALID: in std_logic;	
			--* possible race condition with USB_CLK60G. Reclock at falling edge.
		USB_RXACTIVE: in std_logic;	
			--* possible race condition with USB_CLK60G. Reclock at falling edge.
		USB_RXERROR: in std_logic;	

		USB_DATA_IN: in std_logic_vector(7 downto 0);	
		USB_DATA_OUT: out std_logic_vector(7 downto 0);	

		USB_VBUS_SENSE: in std_logic;
	
		--// DAC interface
		SYNC_B: out std_logic;
		SCLK: out std_logic;
		SDIN: out std_logic;

		--// Monitoring & Control: Atmel controller interface
		UC_AD: inout std_logic_vector(7 downto 0);
		UC_WRN_IN: in std_logic;
		UC_RDN: in std_logic;
		UC_ALE_IN: in std_logic;
		UC_FLASH_CSN: in std_logic;
			-- '0' when uC talks to the flash memory, '1' when otherwise (i.e. possibly
			-- talk to the FPGAs).

		--// Monitoring & Control: Serial ports interface
		-- _L for left connector, _B for bottom connector, _R for right connector.
		-- RX_ signals must be pulled up to signal inactivity on the interface.
		RX_L: in std_logic;
		RX_B: in std_logic;	
		RX_R: in std_logic;	
		TX_L: out std_logic;
		TX_B: out std_logic;	
		TX_R: out std_logic; 

		SERIAL_COM_1 : in std_logic;
		SERIAL_COM_2 : in std_logic;
		--// Test Points
		TEST_POINTS: out std_logic_vector(10 downto 1);
			-- Test points are under the shield. 6 at the edge connector.
		INITB: out std_logic;
			-- easy access test point: INIT#. Easy access with scope.
		DUMMY_OUT: out std_logic
			-- not physically connected to anything
			-- used to prevent alternate function on FPGA pins
			  
			  );
end entity;

architecture Behavioral of COM1400_RX_INTERFACE is
--------------------------------------------------------
--      COMPONENTS
--------------------------------------------------------
component AGC10B is
    port ( 

	--// GLOBAL CLOCKS
   CLK : in std_logic;				

	--// INPUT DATA STREAM
	DATA_I_IN: in std_logic_vector(11 downto 0);
	DATA_Q_IN: in std_logic_vector(11 downto 0);
		-- input samples, 12-bit precision
		-- read at rising edge of CLK when SAMPLE_CLK_IN = '1'.
		-- Format: 2's complement.
	SAMPLE_CLK_IN: in std_logic;
		-- sample clock. 

	--// CONTROLS 
	DECIMATION: in std_logic_vector(23 downto 0);
		-- used to control the AGC loop gain and prevent instabilities.
		-- Adjust as a function of the gain control sensitivity or bandwidth.
		-- minimum 1. 0 is an illegal value
		-- The gain control value AGC_DAC is updated once every DECIMATION input samples. 
   TARGET_MAGNITUDE: in std_logic_vector(13 downto 0);
		-- Use this target to control the nominal A/D converter range used in the average.
		-- May be helpful in cases where the peak to average ratio is high, or the
		-- signal duty cycle is small, or... etc.

 	--// AGC OUTPUTS
	AGC_DAC: out std_logic_vector(9 downto 0);
		-- gain control. May need to be inverted depending on the analog front-end.
		-- Here, maximum gain really means maximum gain.
	AGC_DAC_SAMPLE_CLK: out std_logic;
		-- read AGC_DAC at the rising edge of CLK when AGC_DAC_SAMPLE_CLK = 1.
	
	--// TEST POINTS
	S1: out std_logic_vector(7 downto 0);
	S1_CLK: out std_logic;
	S2: out std_logic_vector(7 downto 0);
	S2_CLK: out std_logic
);
end component;

component USB20 is
    port ( 
	 	--// Clocks Resets
		ASYNC_RESET: in std_logic;
		USB_CLK60G: in std_logic;	
			-- reference clock. 60 MHz.
 		   -- Supplied by the SMSC GT3200 IC (CLK0UT)
			-- Generally not used outside of this component.
			-- Global clock (BUFG MUST be instantiated outside).
		CLK_P: in std_logic;
			-- Main processing or I/O clock used outside of this component.
			-- All application interface signals are synchronous with CLK_p
			-- Key assumption: CLK_P is slightly faster than USB_CLK60G/2.
			-- Other key assumption: CLK_P < 4 * CLK60G

		--// USB PHY interface (SMSC GT3200 IC)
		-- Direct connection between the USB20 component and the USB PHY.
		-- Synchronous with USB_CLK60G clock
		USB_RESET: out std_logic;
		USB_DATABUS16_8: out std_logic;
		USB_SUSPENDN: out std_logic;
		USB_XCVRSELECT: out std_logic;
		USB_TERMSELECT: out std_logic;
		USB_OPMODE: out std_logic_vector(1 downto 0);
			-- operational mode
			-- 00 = normal operation
			-- 01 = non-driving (all terminations removed)
			-- 10 = disable bit stuffing and NRZI encoding (unused)
			-- 11 = reserved (unused)
		USB_LINESTATE: in std_logic_vector(1 downto 0);	
			--* possible race condition with USB_CLK60G. Reclock at falling edge.

		USB_TXVALID: out std_logic;
		USB_TXREADY: in std_logic;	
			--* possible race condition with USB_CLK60G. Reclock at falling edge.
--		USB_VALIDH: inout std_logic;	-- VALIDH is not used in 8-bit mode
		USB_RXVALID: in std_logic;	
			--* possible race condition with USB_CLK60G. Reclock at falling edge.
		USB_RXACTIVE: in std_logic;	
			--* possible race condition with USB_CLK60G. Reclock at falling edge.
		USB_RXERROR: in std_logic;	-- unused

		USB_DATA_IN: in std_logic_vector(7 downto 0);	
			--* possible race condition with USB_CLK60G. Reclock at falling edge.
		USB_DATA_OUT: out std_logic_vector(7 downto 0);	
			-- time critical. User should add OFFSET OUT contraints in the constraint
			-- editor. 

		USB_VBUS_SENSE: in std_logic;
			

		--// Data Stream 1
		-- Synchronous with CLK_P clock
		DATA1_OUT: out std_logic_vector(7 downto 0);
		DATA1_OUT_SAMPLE_CLK: out std_logic;
			-- read DATA1_OUT at rising edge of CLK_P when DATA1_OUT_SAMPLE_CLK = '1'
			-- The latency from a DATA1_OUT_SAMPLE_CLK_REQ input pulse to the DATA1_OUT_SAMPLE_CLK
			-- output pulse is 2 CLK_P.
		DATA1_OUT_BUFFER_EMPTY: out std_logic;
		DATA1_OUT_SAMPLE_CLK_REQ: in std_logic;	
			-- requests data. If no data is available in the buffer, the
			-- DATA1_OUT_SAMPLE_CLK will stay low.
			-- (flow control)
		
		DATA1_IN: in std_logic_vector(7 downto 0);
		DATA1_IN_SAMPLE_CLK: in std_logic;
			-- read DATA1_IN at rising edge of CLK_P when DATA1_IN_SAMPLE_CLK = '1'
		DATA1_IN_SAMPLE_CLK_REQ: out std_logic;
			-- requests data when the input elastic buffer is less than half full. 
			-- (flow control)

		--// Data Stream 2
		-- Synchronous with CLK_P clock
		DATA2_OUT: out std_logic_vector(7 downto 0);
		DATA2_OUT_SAMPLE_CLK: out std_logic;
			-- read DATA2_OUT at rising edge of CLK_P when DATA2_OUT_SAMPLE_CLK = '1'
		DATA2_OUT_BUFFER_EMPTY: out std_logic;
		DATA2_OUT_SAMPLE_CLK_REQ: in std_logic;	
			-- requests data. If no data is available in the buffer, the
			-- DATA2_OUT_SAMPLE_CLK will stay low.
			-- (flow control)
		
		DATA2_IN: in std_logic_vector(7 downto 0);
		DATA2_IN_SAMPLE_CLK: in std_logic;
			-- read DATA2_IN at rising edge of CLK_P when DATA2_IN_SAMPLE_CLK = '1'
		DATA2_IN_SAMPLE_CLK_REQ: out std_logic;
			-- requests data when the input elastic buffer is less than half full. 
			-- (flow control)


		--// Test Points
		USB_TP: out std_logic_vector(10 downto 1)
			-- bit 1: speed after auto-negotiation with host PC: '1' if high-speed.
			-- bit 2: speed after auto-negotiation with host PC: '1' if full-speed.
			-- bit 3: valid SETUP message (PID valid, CRC5 valid). 
			-- 		SETUP is the first message from the host PC to this USB peripheral
			-- bit 4: Host asks to read the descriptor table.
			-- bit 5: data stream 2 input, elastic buffer write pointer LSb (address bit 0)
			-- bit 6: data stream 2 input, elastic buffer read pointer LSb  (address bit 0)
			-- bit 7: data stream 2 input, elastic buffer write pointer MSb (address bit 10)
			-- bit 8: data stream 2 input, elastic buffer read pointer MSb  (address bit 10)
				-- useful in checking flow control
			-- bit 9: data stream 2 output, elastic buffer write pointer MSb (address bit 10)
			-- bit 10: data stream 2 output, elastic buffer read pointer MSb  (address bit 10)
				-- useful in checking flow control

			-- Other useful test points available at the component interface:
			-- VBUS_SENSE. Goes high upon cable being plugged in at both ends
			-- USB_RXERROR: USB PHY detects receive errors 
			-- USB_CLK60G: 60 MHz reference clock from the USB PHY through global buffer.
			-- 				Useful in checking input and output signal timing.
			-- USB_RXVALID from PHY (useful in checking the input timing w.r.t. USB_CLK60G.
			);
end component;

component COMSCOPE is 
	port ( 
	--GLOBAL CLOCKS
  	CLK : in std_logic;				-- Master clock for this FPGA
	CLK_IN: in std_logic;		   -- input signals are synchronous with CLK_IN (could be the same or different than CLK)
	ASYNC_RESET: in std_logic;		-- Asynchronous reset active high


	-- Choose between signed or unsigned representation
	SIGNED_REPRESENTATION_U_SIGNED_N: in std_logic;
		-- '0'  	Unsigned
		-- '1'  	Signed


	-- Input registers
	REG237: in std_logic_vector(7 downto 0);
	REG238: in std_logic_vector(7 downto 0);
	REG239: in std_logic_vector(7 downto 0);
	REG240: in std_logic_vector(7 downto 0);
	REG241: in std_logic_vector(7 downto 0);
	REG242: in std_logic_vector(7 downto 0);
	REG243: in std_logic_vector(7 downto 0);
	REG244: in std_logic_vector(7 downto 0);
	REG245: in std_logic_vector(7 downto 0);
	REG246: in std_logic_vector(7 downto 0);
	REG247: in std_logic_vector(7 downto 0);
	REG248: in std_logic_vector(7 downto 0);
	REG249: in std_logic_vector(7 downto 0);


	-- Trace 1 input signals
	-- 1-bit signals
	SIGNAL_1_BIT_1_1: in std_logic;
	SIGNAL_1_BIT_SAMPLE_CLK_1_1: in std_logic;
	SIGNAL_1_BIT_1_2: in std_logic;
	SIGNAL_1_BIT_SAMPLE_CLK_1_2: in std_logic;
	SIGNAL_1_BIT_1_3: in std_logic;
	SIGNAL_1_BIT_SAMPLE_CLK_1_3: in std_logic;
	SIGNAL_1_BIT_1_4: in std_logic;
	SIGNAL_1_BIT_SAMPLE_CLK_1_4: in std_logic;

	-- 2-bit signals
	SIGNAL_2_BIT_1_1: in std_logic_vector(1 downto 0);
	SIGNAL_2_BIT_SAMPLE_CLK_1_1: in std_logic;
	SIGNAL_2_BIT_1_2: in std_logic_vector(1 downto 0);
	SIGNAL_2_BIT_SAMPLE_CLK_1_2: in std_logic;
	SIGNAL_2_BIT_1_3: in std_logic_vector(1 downto 0);
	SIGNAL_2_BIT_SAMPLE_CLK_1_3: in std_logic;
	SIGNAL_2_BIT_1_4: in std_logic_vector(1 downto 0);
	SIGNAL_2_BIT_SAMPLE_CLK_1_4: in std_logic;

	-- 4-bit signals
	SIGNAL_4_BIT_1_1: in std_logic_vector(3 downto 0);
	SIGNAL_4_BIT_SAMPLE_CLK_1_1: in std_logic;
	SIGNAL_4_BIT_1_2: in std_logic_vector(3 downto 0);
	SIGNAL_4_BIT_SAMPLE_CLK_1_2: in std_logic;
	SIGNAL_4_BIT_1_3: in std_logic_vector(3 downto 0);
	SIGNAL_4_BIT_SAMPLE_CLK_1_3: in std_logic;
	SIGNAL_4_BIT_1_4: in std_logic_vector(3 downto 0);
	SIGNAL_4_BIT_SAMPLE_CLK_1_4: in std_logic;

	-- 8-bit signals
	SIGNAL_8_BIT_1_1: in std_logic_vector(7 downto 0);
	SIGNAL_8_BIT_SAMPLE_CLK_1_1: in std_logic;
	SIGNAL_8_BIT_1_2: in std_logic_vector(7 downto 0);
	SIGNAL_8_BIT_SAMPLE_CLK_1_2: in std_logic;
	SIGNAL_8_BIT_1_3: in std_logic_vector(7 downto 0);
	SIGNAL_8_BIT_SAMPLE_CLK_1_3: in std_logic;
	SIGNAL_8_BIT_1_4: in std_logic_vector(7 downto 0);
	SIGNAL_8_BIT_SAMPLE_CLK_1_4: in std_logic;

	-- 16-bit signals
	SIGNAL_16_BIT_1_1: in std_logic_vector(15 downto 0);
	SIGNAL_16_BIT_SAMPLE_CLK_1_1: in std_logic;
	SIGNAL_16_BIT_1_2: in std_logic_vector(15 downto 0);
	SIGNAL_16_BIT_SAMPLE_CLK_1_2: in std_logic;
	SIGNAL_16_BIT_1_3: in std_logic_vector(15 downto 0);
	SIGNAL_16_BIT_SAMPLE_CLK_1_3: in std_logic;
	SIGNAL_16_BIT_1_4: in std_logic_vector(15 downto 0);
	SIGNAL_16_BIT_SAMPLE_CLK_1_4: in std_logic;
	-- Add signals as necessary (up to 127 signals total per trace)


	-- Trace 2 input signals
	-- 1-bit signals
	SIGNAL_1_BIT_2_1: in std_logic;
	SIGNAL_1_BIT_SAMPLE_CLK_2_1: in std_logic;
	SIGNAL_1_BIT_2_2: in std_logic;
	SIGNAL_1_BIT_SAMPLE_CLK_2_2: in std_logic;
	SIGNAL_1_BIT_2_3: in std_logic;
	SIGNAL_1_BIT_SAMPLE_CLK_2_3: in std_logic;
	SIGNAL_1_BIT_2_4: in std_logic;
	SIGNAL_1_BIT_SAMPLE_CLK_2_4: in std_logic;

	-- 2-bit signals
	SIGNAL_2_BIT_2_1: in std_logic_vector(1 downto 0);
	SIGNAL_2_BIT_SAMPLE_CLK_2_1: in std_logic;
	SIGNAL_2_BIT_2_2: in std_logic_vector(1 downto 0);
	SIGNAL_2_BIT_SAMPLE_CLK_2_2: in std_logic;
	SIGNAL_2_BIT_2_3: in std_logic_vector(1 downto 0);
	SIGNAL_2_BIT_SAMPLE_CLK_2_3: in std_logic;
	SIGNAL_2_BIT_2_4: in std_logic_vector(1 downto 0);
	SIGNAL_2_BIT_SAMPLE_CLK_2_4: in std_logic;

	-- 4-bit signals
	SIGNAL_4_BIT_2_1: in std_logic_vector(3 downto 0);
	SIGNAL_4_BIT_SAMPLE_CLK_2_1: in std_logic;
	SIGNAL_4_BIT_2_2: in std_logic_vector(3 downto 0);
	SIGNAL_4_BIT_SAMPLE_CLK_2_2: in std_logic;
	SIGNAL_4_BIT_2_3: in std_logic_vector(3 downto 0);
	SIGNAL_4_BIT_SAMPLE_CLK_2_3: in std_logic;
	SIGNAL_4_BIT_2_4: in std_logic_vector(3 downto 0);
	SIGNAL_4_BIT_SAMPLE_CLK_2_4: in std_logic;

	-- 8-bit signals
	SIGNAL_8_BIT_2_1: in std_logic_vector(7 downto 0);
	SIGNAL_8_BIT_SAMPLE_CLK_2_1: in std_logic;
	SIGNAL_8_BIT_2_2: in std_logic_vector(7 downto 0);
	SIGNAL_8_BIT_SAMPLE_CLK_2_2: in std_logic;
	SIGNAL_8_BIT_2_3: in std_logic_vector(7 downto 0);
	SIGNAL_8_BIT_SAMPLE_CLK_2_3: in std_logic;
	SIGNAL_8_BIT_2_4: in std_logic_vector(7 downto 0);
	SIGNAL_8_BIT_SAMPLE_CLK_2_4: in std_logic;

	-- 16-bit signals
	SIGNAL_16_BIT_2_1: in std_logic_vector(15 downto 0);
	SIGNAL_16_BIT_SAMPLE_CLK_2_1: in std_logic;
	SIGNAL_16_BIT_2_2: in std_logic_vector(15 downto 0);
	SIGNAL_16_BIT_SAMPLE_CLK_2_2: in std_logic;
	SIGNAL_16_BIT_2_3: in std_logic_vector(15 downto 0);
	SIGNAL_16_BIT_SAMPLE_CLK_2_3: in std_logic;
	SIGNAL_16_BIT_2_4: in std_logic_vector(15 downto 0);
	SIGNAL_16_BIT_SAMPLE_CLK_2_4: in std_logic;
	-- Add signals as necessary (up to 127 signals total per trace)


	-- Trace 3 input signals
	-- 1-bit signals
	SIGNAL_1_BIT_3_1: in std_logic;
	SIGNAL_1_BIT_SAMPLE_CLK_3_1: in std_logic;
	SIGNAL_1_BIT_3_2: in std_logic;
	SIGNAL_1_BIT_SAMPLE_CLK_3_2: in std_logic;
	SIGNAL_1_BIT_3_3: in std_logic;
	SIGNAL_1_BIT_SAMPLE_CLK_3_3: in std_logic;
	SIGNAL_1_BIT_3_4: in std_logic;
	SIGNAL_1_BIT_SAMPLE_CLK_3_4: in std_logic;

	-- 2-bit signals
	SIGNAL_2_BIT_3_1: in std_logic_vector(1 downto 0);
	SIGNAL_2_BIT_SAMPLE_CLK_3_1: in std_logic;
	SIGNAL_2_BIT_3_2: in std_logic_vector(1 downto 0);
	SIGNAL_2_BIT_SAMPLE_CLK_3_2: in std_logic;
	SIGNAL_2_BIT_3_3: in std_logic_vector(1 downto 0);
	SIGNAL_2_BIT_SAMPLE_CLK_3_3: in std_logic;
	SIGNAL_2_BIT_3_4: in std_logic_vector(1 downto 0);
	SIGNAL_2_BIT_SAMPLE_CLK_3_4: in std_logic;

	-- 4-bit signals
	SIGNAL_4_BIT_3_1: in std_logic_vector(3 downto 0);
	SIGNAL_4_BIT_SAMPLE_CLK_3_1: in std_logic;
	SIGNAL_4_BIT_3_2: in std_logic_vector(3 downto 0);
	SIGNAL_4_BIT_SAMPLE_CLK_3_2: in std_logic;
	SIGNAL_4_BIT_3_3: in std_logic_vector(3 downto 0);
	SIGNAL_4_BIT_SAMPLE_CLK_3_3: in std_logic;
	SIGNAL_4_BIT_3_4: in std_logic_vector(3 downto 0);
	SIGNAL_4_BIT_SAMPLE_CLK_3_4: in std_logic;

	-- 8-bit signals
	SIGNAL_8_BIT_3_1: in std_logic_vector(7 downto 0);
	SIGNAL_8_BIT_SAMPLE_CLK_3_1: in std_logic;
	SIGNAL_8_BIT_3_2: in std_logic_vector(7 downto 0);
	SIGNAL_8_BIT_SAMPLE_CLK_3_2: in std_logic;
	SIGNAL_8_BIT_3_3: in std_logic_vector(7 downto 0);
	SIGNAL_8_BIT_SAMPLE_CLK_3_3: in std_logic;
	SIGNAL_8_BIT_3_4: in std_logic_vector(7 downto 0);
	SIGNAL_8_BIT_SAMPLE_CLK_3_4: in std_logic;

	-- 16-bit signals
	SIGNAL_16_BIT_3_1: in std_logic_vector(15 downto 0);
	SIGNAL_16_BIT_SAMPLE_CLK_3_1: in std_logic;
	SIGNAL_16_BIT_3_2: in std_logic_vector(15 downto 0);
	SIGNAL_16_BIT_SAMPLE_CLK_3_2: in std_logic;
	SIGNAL_16_BIT_3_3: in std_logic_vector(15 downto 0);
	SIGNAL_16_BIT_SAMPLE_CLK_3_3: in std_logic;
	SIGNAL_16_BIT_3_4: in std_logic_vector(15 downto 0);
	SIGNAL_16_BIT_SAMPLE_CLK_3_4: in std_logic;
	-- Add signals as necessary (up to 127 signals total per trace)


	-- Trace 4 input signals
	-- 1-bit signals
	SIGNAL_1_BIT_4_1: in std_logic;
	SIGNAL_1_BIT_SAMPLE_CLK_4_1: in std_logic;
	SIGNAL_1_BIT_4_2: in std_logic;
	SIGNAL_1_BIT_SAMPLE_CLK_4_2: in std_logic;
	SIGNAL_1_BIT_4_3: in std_logic;
	SIGNAL_1_BIT_SAMPLE_CLK_4_3: in std_logic;
	SIGNAL_1_BIT_4_4: in std_logic;
	SIGNAL_1_BIT_SAMPLE_CLK_4_4: in std_logic;

	-- 2-bit signals
	SIGNAL_2_BIT_4_1: in std_logic_vector(1 downto 0);
	SIGNAL_2_BIT_SAMPLE_CLK_4_1: in std_logic;
	SIGNAL_2_BIT_4_2: in std_logic_vector(1 downto 0);
	SIGNAL_2_BIT_SAMPLE_CLK_4_2: in std_logic;
	SIGNAL_2_BIT_4_3: in std_logic_vector(1 downto 0);
	SIGNAL_2_BIT_SAMPLE_CLK_4_3: in std_logic;
	SIGNAL_2_BIT_4_4: in std_logic_vector(1 downto 0);
	SIGNAL_2_BIT_SAMPLE_CLK_4_4: in std_logic;

	-- 4-bit signals
	SIGNAL_4_BIT_4_1: in std_logic_vector(3 downto 0);
	SIGNAL_4_BIT_SAMPLE_CLK_4_1: in std_logic;
	SIGNAL_4_BIT_4_2: in std_logic_vector(3 downto 0);
	SIGNAL_4_BIT_SAMPLE_CLK_4_2: in std_logic;
	SIGNAL_4_BIT_4_3: in std_logic_vector(3 downto 0);
	SIGNAL_4_BIT_SAMPLE_CLK_4_3: in std_logic;
	SIGNAL_4_BIT_4_4: in std_logic_vector(3 downto 0);
	SIGNAL_4_BIT_SAMPLE_CLK_4_4: in std_logic;

	-- 8-bit signals
	SIGNAL_8_BIT_4_1: in std_logic_vector(7 downto 0);
	SIGNAL_8_BIT_SAMPLE_CLK_4_1: in std_logic;
	SIGNAL_8_BIT_4_2: in std_logic_vector(7 downto 0);
	SIGNAL_8_BIT_SAMPLE_CLK_4_2: in std_logic;
	SIGNAL_8_BIT_4_3: in std_logic_vector(7 downto 0);
	SIGNAL_8_BIT_SAMPLE_CLK_4_3: in std_logic;
	SIGNAL_8_BIT_4_4: in std_logic_vector(7 downto 0);
	SIGNAL_8_BIT_SAMPLE_CLK_4_4: in std_logic;

	-- 16-bit signals
	SIGNAL_16_BIT_4_1: in std_logic_vector(15 downto 0);
	SIGNAL_16_BIT_SAMPLE_CLK_4_1: in std_logic;
	SIGNAL_16_BIT_4_2: in std_logic_vector(15 downto 0);
	SIGNAL_16_BIT_SAMPLE_CLK_4_2: in std_logic;
	SIGNAL_16_BIT_4_3: in std_logic_vector(15 downto 0);
	SIGNAL_16_BIT_SAMPLE_CLK_4_3: in std_logic;
	SIGNAL_16_BIT_4_4: in std_logic_vector(15 downto 0);
	SIGNAL_16_BIT_SAMPLE_CLK_4_4: in std_logic;
	-- Add signals as necessary (up to 127 signals total per trace)



	-- Trigger signals
	-- 1-bit Triggers
	TRIGGER_1_BIT_1: in std_logic;
	TRIGGER_1_BIT_SAMPLE_CLK_1: in std_logic;
	TRIGGER_1_BIT_2: in std_logic;
	TRIGGER_1_BIT_SAMPLE_CLK_2: in std_logic;
	TRIGGER_1_BIT_3: in std_logic;
	TRIGGER_1_BIT_SAMPLE_CLK_3: in std_logic;
	TRIGGER_1_BIT_4: in std_logic;
	TRIGGER_1_BIT_SAMPLE_CLK_4: in std_logic;

	-- 2-bit Triggers
	TRIGGER_2_BIT_1: in std_logic_vector(1 downto 0);
	TRIGGER_2_BIT_SAMPLE_CLK_1: in std_logic;
	TRIGGER_2_BIT_2: in std_logic_vector(1 downto 0);
	TRIGGER_2_BIT_SAMPLE_CLK_2: in std_logic;
	TRIGGER_2_BIT_3: in std_logic_vector(1 downto 0);
	TRIGGER_2_BIT_SAMPLE_CLK_3: in std_logic;
	TRIGGER_2_BIT_4: in std_logic_vector(1 downto 0);
	TRIGGER_2_BIT_SAMPLE_CLK_4: in std_logic;

	-- 4-bit Triggers
	TRIGGER_4_BIT_1: in std_logic_vector(3 downto 0);
	TRIGGER_4_BIT_SAMPLE_CLK_1: in std_logic;
	TRIGGER_4_BIT_2: in std_logic_vector(3 downto 0);
	TRIGGER_4_BIT_SAMPLE_CLK_2: in std_logic;
	TRIGGER_4_BIT_3: in std_logic_vector(3 downto 0);
	TRIGGER_4_BIT_SAMPLE_CLK_3: in std_logic;
	TRIGGER_4_BIT_4: in std_logic_vector(3 downto 0);
	TRIGGER_4_BIT_SAMPLE_CLK_4: in std_logic;

	-- 8-bit Triggers
	TRIGGER_8_BIT_1: in std_logic_vector(7 downto 0);
	TRIGGER_8_BIT_SAMPLE_CLK_1: in std_logic;
	TRIGGER_8_BIT_2: in std_logic_vector(7 downto 0);
	TRIGGER_8_BIT_SAMPLE_CLK_2: in std_logic;
	TRIGGER_8_BIT_3: in std_logic_vector(7 downto 0);
	TRIGGER_8_BIT_SAMPLE_CLK_3: in std_logic;
	TRIGGER_8_BIT_4: in std_logic_vector(7 downto 0);
	TRIGGER_8_BIT_SAMPLE_CLK_4: in std_logic;

	-- 16-bit Triggers
	TRIGGER_16_BIT_1: in std_logic_vector(15 downto 0);
	TRIGGER_16_BIT_SAMPLE_CLK_1: in std_logic;
	TRIGGER_16_BIT_2: in std_logic_vector(15 downto 0);
	TRIGGER_16_BIT_SAMPLE_CLK_2: in std_logic;
	TRIGGER_16_BIT_3: in std_logic_vector(15 downto 0);
	TRIGGER_16_BIT_SAMPLE_CLK_3: in std_logic;
	TRIGGER_16_BIT_4: in std_logic_vector(15 downto 0);
	TRIGGER_16_BIT_SAMPLE_CLK_4: in std_logic;
	-- Add triggers as necessary (up to 127 trigger signals total)


	-- Output registers
	REG250: out std_logic_vector(7 downto 0);
	REG251: out std_logic_vector(7 downto 0);

	-- Other input signals
	TRIGGER_REARM_TOGGLE: in std_logic;
	FORCE_TRIGGER_TOGGLE: in std_logic;
	REG250_READ: in std_logic;
	START_CAPTURE_TOGGLE: in std_logic
);
end component;

component com1400_1
port(
	CLKIN_IN        : in    std_logic; 
   RST_IN          : in    std_logic; 
   CLK0G	        : out   std_logic; 
   CLK180G	        : out   std_logic; 
   LOCKED_OUT      : out   std_logic
	);
end component;

component com1400_3
port(
	CLKIN_IN : IN std_logic;
	RST_IN : IN std_logic;          
	CLKFX_OUT : OUT std_logic;
	LOCKED_OUT : OUT std_logic
	);
end component;

component DAC is port ( 

	--// Clock
	CLK : in std_logic;		-- Main internal processing clock
	
	--// User Interface
	DATA_IN: in std_logic_vector(9 downto 0);
		-- aligned with DATA_IN_CLK
	DATA_IN_CLK: in std_logic;
		-- 1 CLK pulse indicating that DATA_IN is valid.

	OP_MODE: in std_logic_vector(1 downto 0);
		-- Operation Mode aligned with DATA_IN_CLK
		-- 0 0 - Normal operation
		-- Power-down modes:  
		-- 0 1 - 1 kOhm to GND
		-- 1 0 - 100 kOhm to GND
		-- 1 1 - Three-state

	DONE: out std_logic;
		-- indicates DAC availability	(1 CLK early)
		-- '1' indicates interface is available on NEXT clk. 
		-- goes to '0' immediately after valid DATA_IN_CLK pulse/level
		-- is presented, used for flow control

	--// DAC interface
	SYNC_B: out std_logic;
	SCLK: out std_logic;	
		-- Maximum frequency of 30 MHz.
		-- doesn't toggle when DAC is inactive
	SDIN: out std_logic
);
end component;

--------------------------------------------------------
--     ATTRIBUTES
--------------------------------------------------------


-- Pull unused input data pins low in constraint file
--attribute PULLDOWN of LEFT_CONNECTOR<3>: signal is "YES";
-- PULL DOWN INPUT SIGNALS
-- PULL UP INPUT SAMPLING CLOCK AS PER SPECS
--------------------------------------------------------
--     SIGNALS
--------------------------------------------------------
-- Suffix _D indicates a one CLK delayed version of the net with the same name
-- Suffix _E indicates an extended precision version of the net with the same name
-- Suffix _N indicates an inverted version of the net with the same name

--// CLOCKS, RESETS
signal USB_CLK60G: std_logic;
signal USB_CLK60B: std_logic;
signal CLK_P: std_logic;
signal CLK_IN0G: std_logic;
signal CLK_IN180G: std_logic;
signal DCM_LOCKED_001: std_logic;
signal DCM_LOCKED_003: std_logic;
signal COUNTER8:  std_logic_vector(2 downto 0);
signal ASYNC_RESET_DCM: std_logic;
signal ASYNC_RESET: std_logic;
	-- asynchronous reset
signal RESET_COUNTER: std_logic_vector(17 downto 0) := "000000000000000000";	-- initialize for simulations
	-- spans 2^17/60MHz = 2.18ms

--// INPUT SIGNALS
signal DATA_IN_0: std_logic_vector(11 downto 0);
signal DATA_IN_0_D: std_logic_vector(11 downto 0);
signal DATA_IN_1F: std_logic_vector(11 downto 0);
signal DATA_IN_1: std_logic_vector(11 downto 0);
signal DATA_IN_1_D: std_logic_vector(11 downto 0);
signal DATA_IN_2: std_logic_vector(11 downto 0);
signal DATA_IN_2_D: std_logic_vector(11 downto 0);
signal DATA_IN_3F: std_logic_vector(11 downto 0);
signal DATA_IN_3: std_logic_vector(11 downto 0);
signal DATA_IN_3_D: std_logic_vector(11 downto 0);

--// AGC
constant AGC_TARGET_MAGNITUDE: std_logic_vector(13 downto 0) := "00110000000000";
	-- Set the AGC_TARGET_MAGNITUDE empirically for each type of demodulator.
	-- For constant amplitude modulations, one can go close to the maximum positive ceiling.
signal AGC_DECIMATION: std_logic_vector(23 downto 0);
signal AGC_INPUT_SAMPLING_RATE: std_logic;
signal AGC_UPDATE_RATE: std_logic_vector(4 downto 0);
signal DAC_READY: std_logic;
signal AGC_DAC_SAMPLE_CLK: std_logic;
signal AGC_DAC_SAMPLE_CLKB: std_logic;
signal AGC_DAC: std_logic_vector(9 downto 0);

--// USB Interface for M&C
signal RX_USB: std_logic_vector(7 downto 0); 
signal RX_USB_SAMPLE_CLK_REQ: std_logic;
signal RX_USB_BUFFER_EMPTY: std_logic;
signal RX_USB_BUFFER_EMPTY_D: std_logic;
signal RX_USB_BUFFER_EMPTY_D2: std_logic;
signal TX_USB: std_logic_vector(7 downto 0); 
signal TX_USB_SAMPLE_CLK: std_logic;
signal TX_USB_TOGGLE: std_logic;
signal TX_USB_TOGGLE_D: std_logic;
signal TX_USB_TOGGLE_D2: std_logic;
signal USB_TP: std_logic_vector(10 downto 1);

--// ComScope traces
signal S1_1: std_logic_vector(7 downto 0);	
signal S1_1_CLK: std_logic;
signal S1_1B_CLK: std_logic;
signal S1_2: std_logic_vector(7 downto 0);	
signal S1_2_CLK: std_logic;
signal S1_2B_CLK: std_logic;
signal S1_3: std_logic_vector(7 downto 0);	
signal S1_3_CLK: std_logic;
signal S1_3B_CLK: std_logic;
signal S1_4: std_logic_vector(7 downto 0);	
signal S1_4_CLK: std_logic;
signal S1_4B_CLK: std_logic;

signal S2_1: std_logic_vector(7 downto 0);	
signal S2_1_CLK: std_logic;
signal S2_1B_CLK: std_logic;
signal S2_2: std_logic_vector(7 downto 0);	
signal S2_2_CLK: std_logic;
signal S2_2B_CLK: std_logic;
signal S2_3: std_logic_vector(7 downto 0);	
signal S2_3_CLK: std_logic;
signal S2_3B_CLK: std_logic;
signal S2_4: std_logic_vector(7 downto 0);	
signal S2_4_CLK: std_logic;
signal S2_4B_CLK: std_logic;

signal T1: std_logic;
signal T1_CLK: std_logic;
signal T2: std_logic;
signal T2_CLK: std_logic;

--// TEST POINTS
signal TP1: std_logic_vector(10 downto 1);
signal TP2: std_logic_vector(10 downto 1);

signal TRIGGER_REARM_TOGGLE: std_logic:= '0';
signal FORCE_TRIGGER_TOGGLE: std_logic:= '0';
signal START_CAPTURE_TOGGLE: std_logic:= '0';
signal REG250_READ: std_logic;

--// Atmel uc interface
signal UC_ADDRESS: std_logic_vector(7 downto 0);
signal UC_ADDR_D: std_logic_vector(7 downto 0);
signal UC_ADDR_D2: std_logic_vector(7 downto 0);
signal UC_RDN_D: std_logic;
signal UC_RDN_D2: std_logic;
signal CONFIG_CHANGE_TOGGLE: std_logic;
signal CONFIG_CHANGE_TOGGLE_D: std_logic;
signal CONFIG_CHANGE_TOGGLE_D2: std_logic;
signal UC_ALE: std_logic;
signal UC_ALE_B: std_logic;
signal UC_WRN: std_logic;
signal UC_WRN_B: std_logic;

--// Control Registers
signal REG0: std_logic_vector(7 downto 0);
signal REG255: std_logic_vector(7 downto 0);

--// Status Registers
signal SREG0: std_logic_vector(7 downto 0);

--// ComScope control registers
signal REG237: std_logic_vector(7 downto 0):= "00000000";
signal REG238: std_logic_vector(7 downto 0):= "00000000";
signal REG239: std_logic_vector(7 downto 0):= "00000000";
signal REG240: std_logic_vector(7 downto 0):= "00001101";	-- Trace 1 signal selection
signal REG241: std_logic_vector(7 downto 0):= "00000000";	-- Trigger position, decimation
signal REG242: std_logic_vector(7 downto 0):= "00001110";	-- Trace 2 signal selection
signal REG243: std_logic_vector(7 downto 0):= "00000000";	-- Trigger position, decimation
signal REG244: std_logic_vector(7 downto 0):= "00000000";	-- Trace 3 signal selection
signal REG245: std_logic_vector(7 downto 0):= "00000000";	-- Trigger position, decimation
signal REG246: std_logic_vector(7 downto 0):= "00000000";	-- Trace 4 signal selection
signal REG247: std_logic_vector(7 downto 0):= "00000000";	-- Trigger position, decimation
signal REG248: std_logic_vector(7 downto 0):= "10001101";	-- Trigger edge, trigger signal
signal REG249: std_logic_vector(7 downto 0):= "10111111";	-- Trigger threshold
signal REG250: std_logic_vector(7 downto 0):= "00000000";
signal REG251: std_logic_vector(7 downto 0):= "00000000";

--// FPGA option and revision (ASCII format)
-- These two ASCII characters appear in the Comblock Control Center.
-- Very helpful in identifying which firmware version is currently running
-- in the FPGA. 
-- '0' = 48
-- 'A' = 65
constant OPTION: std_logic_vector(7 downto 0) := x"55";	-- 'U'
constant REVISION: std_logic_vector(7 downto 0) := x"31";	-- '1'
--------------------------------------------------------
--      IMPLEMENTATION
--------------------------------------------------------

begin
--// CLOCKS, RESET ---------------------
-- Generate miscellaneous glocal clocks and resets from the 
-- reference 60 MHz clock (from the USB PHY). 
-- FOR MOST APPLICATIONS, KEEP THIS SECTION UNCHANGED.

-- Generate two types of asynchronous resets:
-- one (ASYNC_RESET_DCM) to hold the DCMs in reset for about 1ms after configuration
-- while the USB PHY switches from 30 MHz clock to 60 MHz clock.
-- The other (ASYNC_RESET) to wait until the DCM stabilize to their
-- desired frequencies. Takes about 100 uS.

RESET_COUNTER_GEN_001: process(USB_CLK60G)
begin
	if rising_edge(USB_CLK60G) then
		ASYNC_RESET <= not(RESET_COUNTER(17));	-- about 2 ms.

		if(RESET_COUNTER(17) = '0') then
			RESET_COUNTER <= RESET_COUNTER + 1;
	  	end if;

		-- Reset is synchronous with USB_CLK60G but asynchronous with CLK_P and CLK_IO.
		-- reclocked to prevent glitches (would reset DCM).
		if (RESET_COUNTER(17 downto 16) = "00") then
			ASYNC_RESET_DCM <= '1';	-- about 1ms
		else
			ASYNC_RESET_DCM <= '0';	-- about 1ms
		end if;			
  	end if;
end process;

-- force selection of correct IBUF
IBUF_001: IBUF port map(I => UC_WRN_IN, O => UC_WRN_B);  
IBUF_002: IBUF port map(I => UC_ALE_IN, O => UC_ALE_B); 
IBUF_003: IBUF port map(I => USB_CLK60, O => USB_CLK60B); 

-- Global buffers
BUFG_001: BUFG  port map (I=> USB_CLK60B, O=> USB_CLK60G);	
	-- reference clock from USB PHY. 
BUFG_006: BUFG port map(I => UC_WRN_B, O => UC_WRN);  
BUFG_007: BUFG port map(I => UC_ALE_B, O => UC_ALE); 

-- external input clock 
-- DCM used to clean CLK_IN and restore a 50% duty cycle
-- No offset.
Inst_com1400_1: com1400_1 PORT MAP(
	CLKIN_IN => LEFT_CONNECTOR(1),  	-- IBUFG within
	RST_IN => ASYNC_RESET_DCM,
	CLK0G => CLK_IN0G,					-- global clock. BUFG within
	CLK180G => CLK_IN180G,				-- global clock. BUFG within
	LOCKED_OUT => DCM_LOCKED_001
);


-- DCM used as frequency synthesizer for built-in test generator clock. 60 MHz x M / D = 80 MHz, 
-- where M = 4, D = 3. M and D are set through the attributes above.
Inst_com1400_3: com1400_3 port map(
	CLKIN_IN => USB_CLK60G,
	RST_IN => ASYNC_RESET_DCM,
	CLKFX_OUT => CLK_P,
	LOCKED_OUT => DCM_LOCKED_003
);



-- Test clocks. Generate a divide-by-8 clock and forward to 
-- the INIT# test point (located next to the 'DONE' test point)
-- This is an easy way to verify that the VHDL code is properly implemented
-- and that no glitches exist on an external or internal clock.
process(CLK_P)
begin
	if rising_edge(CLK_P) then
		COUNTER8 <= COUNTER8 + 1;
	end if;
end process;
INITB <= COUNTER8(2);

--// USB INTERFACE ---------------------
-- The USB component is available in .ngc form (not source code).
-- Two virtual channels are available:
-- Channel 1 for Monitoring and Control. DO NOT CHANGE.
-- Channel 2 is a payload channel (available for custom applications). 
-- Tie the channel 2 inputs and outputs to the application input and 
-- output streams as needed.
USB20_001: USB20 port map(
	USB_CLK60G => USB_CLK60G,
	ASYNC_RESET => ASYNC_RESET,
	CLK_P => CLK_P,

	-- interface with USB PHY. Direct connection to I/O pins.
	USB_RESET => USB_RESET,
--	USB_DATABUS16_8 => USB_DATABUS16_8,
--	USB_SUSPENDN => USB_SUSPENDN,
	USB_XCVRSELECT => USB_XCVRSELECT,
	USB_TERMSELECT => USB_TERMSELECT,
	USB_OPMODE => USB_OPMODE,
	USB_LINESTATE => USB_LINESTATE,
	USB_TXVALID => USB_TXVALID,
	USB_TXREADY => USB_TXREADY,
	USB_RXVALID => USB_RXVALID,
	USB_RXACTIVE => USB_RXACTIVE,
	USB_RXERROR => USB_RXERROR,
	USB_DATA_IN => USB_DATA_IN,
	USB_DATA_OUT => USB_DATA_OUT,
	USB_VBUS_SENSE => USB_VBUS_SENSE,

	-- M&C channel. Interfaces with Atmel uC. Uses data stream 1
	DATA1_OUT => RX_USB,
	DATA1_OUT_SAMPLE_CLK => open,
	DATA1_OUT_BUFFER_EMPTY => RX_USB_BUFFER_EMPTY,
	DATA1_OUT_SAMPLE_CLK_REQ => RX_USB_SAMPLE_CLK_REQ,
	DATA1_IN => TX_USB,
   DATA1_IN_SAMPLE_CLK => TX_USB_SAMPLE_CLK,
	DATA1_IN_SAMPLE_CLK_REQ => open,	
		-- Atmel uC is much slower than USB. no need to verify tx flow control.

	-- Payload channel. Data stream 2
	DATA2_OUT => open,
	DATA2_OUT_SAMPLE_CLK => open,
	DATA2_OUT_BUFFER_EMPTY => open,
	DATA2_OUT_SAMPLE_CLK_REQ => '0',
	DATA2_IN => x"00",
	DATA2_IN_SAMPLE_CLK => '0',
	DATA2_IN_SAMPLE_CLK_REQ => open,
	USB_TP => USB_TP
);


--// DAC DRIVER ---------------------
-- A 10-bit DAC/Analog output is built-in for gain control. 
-- The DAC output is connected to pin B13 of the J5 connector. 
-- The DAC can be disabled in software to a high impedance state or 
-- physically disabled by removing the 0. R5 resistor. The DAC can 
-- typically operate at a 0.5V/sec slew rate. The output voltage range 
-- is from 0 to 3.3 V. The driver provided in the code template allows 
-- for approximate refresh rate of up to 1.765 MHz (slew rate limited.)

AGC_DAC_SAMPLE_CLKB <= AGC_DAC_SAMPLE_CLK and DAC_READY;

DAC_001: DAC port map(		
	CLK => CLK_IN0G,
	--// User Interface
	DATA_IN => AGC_DAC,
	DATA_IN_CLK => AGC_DAC_SAMPLE_CLKB,
	OP_MODE => "00",
	DONE => DAC_READY,
  	--// DAC interface
  	SYNC_B => SYNC_B,
  	SCLK => SCLK,
  	SDIN => SDIN
);		 


--// RECLOCK INPUT SIGNALS  2*10-bit DDR INTERFACE -----------------
-- Demultiplex the COM-3008 high-speed DDR interface
-- Two successive samples are collected DATA_IN_0/DATA_IN_1.
-- These samples will be processed in parallel as the Spartan-3 is not
-- capable of doing any significant signal processing at the maximum 210 MSamples/s.
-- Convert to 2's complement.

IFDDRRSE_001 : IFDDRRSE
port map (
	Q0 => DATA_IN_0(11),    -- Posedge data output
	Q1 => DATA_IN_1(11),    -- Negedge data output
	C0 => CLK_IN0G,    
	C1 => CLK_IN180G,   
	CE => '1',    -- Clock enable input
	D => LEFT_CONNECTOR(3),      -- Data input (connect directly to top-level port)
	R => '0',      -- Synchronous reset input
	S => '0'       -- Synchronous preset input
);

IFDDRRSE_002 : IFDDRRSE
port map (
	Q0 => DATA_IN_0(10),    -- Posedge data output
	Q1 => DATA_IN_1(10),    -- Negedge data output
	C0 => CLK_IN0G,    
	C1 => CLK_IN180G,   
	CE => '1',    -- Clock enable input
	D => LEFT_CONNECTOR(4),      -- Data input (connect directly to top-level port)
	R => '0',      -- Synchronous reset input
	S => '0'       -- Synchronous preset input
);

IFDDRRSE_003 : IFDDRRSE
port map (
	Q0 => DATA_IN_0(9),    -- Posedge data output
	Q1 => DATA_IN_1(9),    -- Negedge data output
	C0 => CLK_IN0G,    
	C1 => CLK_IN180G,   
	CE => '1',    -- Clock enable input
	D => LEFT_CONNECTOR(5),      -- Data input (connect directly to top-level port)
	R => '0',      -- Synchronous reset input
	S => '0'       -- Synchronous preset input
);

IFDDRRSE_004 : IFDDRRSE
port map (
	Q0 => DATA_IN_0(8),    -- Posedge data output
	Q1 => DATA_IN_1(8),    -- Negedge data output
	C0 => CLK_IN0G,    
	C1 => CLK_IN180G,   
	CE => '1',    -- Clock enable input
	D => LEFT_CONNECTOR(6),      -- Data input (connect directly to top-level port)
	R => '0',      -- Synchronous reset input
	S => '0'       -- Synchronous preset input
);

IFDDRRSE_005 : IFDDRRSE
port map (
	Q0 => DATA_IN_0(7),    -- Posedge data output
	Q1 => DATA_IN_1(7),    -- Negedge data output
	C0 => CLK_IN0G,    
	C1 => CLK_IN180G,   
	CE => '1',    -- Clock enable input
	D => LEFT_CONNECTOR(7),      -- Data input (connect directly to top-level port)
	R => '0',      -- Synchronous reset input
	S => '0'       -- Synchronous preset input
);

IFDDRRSE_006 : IFDDRRSE
port map (
	Q0 => DATA_IN_0(6),    -- Posedge data output
	Q1 => DATA_IN_1(6),    -- Negedge data output
	C0 => CLK_IN0G,    
	C1 => CLK_IN180G,   
	CE => '1',    -- Clock enable input
	D => LEFT_CONNECTOR(8),      -- Data input (connect directly to top-level port)
	R => '0',      -- Synchronous reset input
	S => '0'       -- Synchronous preset input
);

IFDDRRSE_007 : IFDDRRSE
port map (
	Q0 => DATA_IN_0(5),    -- Posedge data output
	Q1 => DATA_IN_1(5),    -- Negedge data output
	C0 => CLK_IN0G,    
	C1 => CLK_IN180G,   
	CE => '1',    -- Clock enable input
	D => LEFT_CONNECTOR(9),      -- Data input (connect directly to top-level port)
	R => '0',      -- Synchronous reset input
	S => '0'       -- Synchronous preset input
);

IFDDRRSE_008 : IFDDRRSE
port map (
	Q0 => DATA_IN_0(4),    -- Posedge data output
	Q1 => DATA_IN_1(4),    -- Negedge data output
	C0 => CLK_IN0G,    
	C1 => CLK_IN180G,   
	CE => '1',    -- Clock enable input
	D => LEFT_CONNECTOR(11),      -- Data input (connect directly to top-level port)
	R => '0',      -- Synchronous reset input
	S => '0'       -- Synchronous preset input
);

IFDDRRSE_009 : IFDDRRSE
port map (
	Q0 => DATA_IN_0(3),    -- Posedge data output
	Q1 => DATA_IN_1(3),    -- Negedge data output
	C0 => CLK_IN0G,    
	C1 => CLK_IN180G,   
	CE => '1',    -- Clock enable input
	D => LEFT_CONNECTOR(12),      -- Data input (connect directly to top-level port)
	R => '0',      -- Synchronous reset input
	S => '0'       -- Synchronous preset input
);

IFDDRRSE_010 : IFDDRRSE
port map (
	Q0 => DATA_IN_0(2),    -- Posedge data output
	Q1 => DATA_IN_1(2),    -- Negedge data output
	C0 => CLK_IN0G,    
	C1 => CLK_IN180G,   
	CE => '1',    -- Clock enable input
	D => LEFT_CONNECTOR(13),      -- Data input (connect directly to top-level port)
	R => '0',      -- Synchronous reset input
	S => '0'       -- Synchronous preset input
);

IFDDRRSE_011 : IFDDRRSE
port map (
	Q0 => DATA_IN_0(1),    -- Posedge data output
	Q1 => DATA_IN_1(1),    -- Negedge data output
	C0 => CLK_IN0G,    
	C1 => CLK_IN180G,   
	CE => '1',    -- Clock enable input
	D => LEFT_CONNECTOR(28),      -- Data input (connect directly to top-level port)
	R => '0',      -- Synchronous reset input
	S => '0'       -- Synchronous preset input
);

IFDDRRSE_012 : IFDDRRSE
port map (
	Q0 => DATA_IN_0(0),    -- Posedge data output
	Q1 => DATA_IN_1(0),    -- Negedge data output
	C0 => CLK_IN0G,    
	C1 => CLK_IN180G,   
	CE => '1',    -- Clock enable input
	D => LEFT_CONNECTOR(29),      -- Data input (connect directly to top-level port)
	R => '0',      -- Synchronous reset input
	S => '0'       -- Synchronous preset input
);

------------------------------------------------------

IFDDRRSE_021 : IFDDRRSE
port map (
	Q0 => DATA_IN_2(11),    -- Posedge data output
	Q1 => DATA_IN_3(11),    -- Negedge data output
	C0 => CLK_IN0G,    
	C1 => CLK_IN180G,   
	CE => '1',    -- Clock enable input
	D => LEFT_CONNECTOR(14),      -- Data input (connect directly to top-level port)
	R => '0',      -- Synchronous reset input
	S => '0'       -- Synchronous preset input
);

IFDDRRSE_022 : IFDDRRSE
port map (
	Q0 => DATA_IN_2(10),    -- Posedge data output
	Q1 => DATA_IN_3(10),    -- Negedge data output
	C0 => CLK_IN0G,    
	C1 => CLK_IN180G,   
	CE => '1',    -- Clock enable input
	D => LEFT_CONNECTOR(15),      -- Data input (connect directly to top-level port)
	R => '0',      -- Synchronous reset input
	S => '0'       -- Synchronous preset input
);

IFDDRRSE_023 : IFDDRRSE
port map (
	Q0 => DATA_IN_2(9),    -- Posedge data output
	Q1 => DATA_IN_3(9),    -- Negedge data output
	C0 => CLK_IN0G,    
	C1 => CLK_IN180G,   
	CE => '1',    -- Clock enable input
	D => LEFT_CONNECTOR(16),      -- Data input (connect directly to top-level port)
	R => '0',      -- Synchronous reset input
	S => '0'       -- Synchronous preset input
);

IFDDRRSE_024 : IFDDRRSE
port map (
	Q0 => DATA_IN_2(8),    -- Posedge data output
	Q1 => DATA_IN_3(8),    -- Negedge data output
	C0 => CLK_IN0G,    
	C1 => CLK_IN180G,   
	CE => '1',    -- Clock enable input
	D => LEFT_CONNECTOR(17),      -- Data input (connect directly to top-level port)
	R => '0',      -- Synchronous reset input
	S => '0'       -- Synchronous preset input
);

IFDDRRSE_025 : IFDDRRSE
port map (
	Q0 => DATA_IN_2(7),    -- Posedge data output
	Q1 => DATA_IN_3(7),    -- Negedge data output
	C0 => CLK_IN0G,    
	C1 => CLK_IN180G,   
	CE => '1',    -- Clock enable input
	D => LEFT_CONNECTOR(18),      -- Data input (connect directly to top-level port)
	R => '0',      -- Synchronous reset input
	S => '0'       -- Synchronous preset input
);

IFDDRRSE_026 : IFDDRRSE
port map (
	Q0 => DATA_IN_2(6),    -- Posedge data output
	Q1 => DATA_IN_3(6),    -- Negedge data output
	C0 => CLK_IN0G,    
	C1 => CLK_IN180G,   
	CE => '1',    -- Clock enable input
	D => LEFT_CONNECTOR(19),      -- Data input (connect directly to top-level port)
	R => '0',      -- Synchronous reset input
	S => '0'       -- Synchronous preset input
);

IFDDRRSE_027 : IFDDRRSE
port map (
	Q0 => DATA_IN_2(5),    -- Posedge data output
	Q1 => DATA_IN_3(5),    -- Negedge data output
	C0 => CLK_IN0G,    
	C1 => CLK_IN180G,   
	CE => '1',    -- Clock enable input
	D => LEFT_CONNECTOR(21),      -- Data input (connect directly to top-level port)
	R => '0',      -- Synchronous reset input
	S => '0'       -- Synchronous preset input
);

IFDDRRSE_028 : IFDDRRSE
port map (
	Q0 => DATA_IN_2(4),    -- Posedge data output
	Q1 => DATA_IN_3(4),    -- Negedge data output
	C0 => CLK_IN0G,    
	C1 => CLK_IN180G,   
	CE => '1',    -- Clock enable input
	D => LEFT_CONNECTOR(22),      -- Data input (connect directly to top-level port)
	R => '0',      -- Synchronous reset input
	S => '0'       -- Synchronous preset input
);

IFDDRRSE_029 : IFDDRRSE
port map (
	Q0 => DATA_IN_2(3),    -- Posedge data output
	Q1 => DATA_IN_3(3),    -- Negedge data output
	C0 => CLK_IN0G,    
	C1 => CLK_IN180G,   
	CE => '1',    -- Clock enable input
	D => LEFT_CONNECTOR(23),      -- Data input (connect directly to top-level port)
	R => '0',      -- Synchronous reset input
	S => '0'       -- Synchronous preset input
);

IFDDRRSE_030 : IFDDRRSE
port map (
	Q0 => DATA_IN_2(2),    -- Posedge data output
	Q1 => DATA_IN_3(2),    -- Negedge data output
	C0 => CLK_IN0G,    
	C1 => CLK_IN180G,   
	CE => '1',    -- Clock enable input
	D => LEFT_CONNECTOR(24),      -- Data input (connect directly to top-level port)
	R => '0',      -- Synchronous reset input
	S => '0'       -- Synchronous preset input
);

IFDDRRSE_031 : IFDDRRSE
port map (
	Q0 => DATA_IN_2(1),    -- Posedge data output
	Q1 => DATA_IN_3(1),    -- Negedge data output
	C0 => CLK_IN0G,    
	C1 => CLK_IN180G,   
	CE => '1',    -- Clock enable input
	D => LEFT_CONNECTOR(31),      -- Data input (connect directly to top-level port)
	R => '0',      -- Synchronous reset input
	S => '0'       -- Synchronous preset input
);

IFDDRRSE_032 : IFDDRRSE
port map (
	Q0 => DATA_IN_2(0),    -- Posedge data output
	Q1 => DATA_IN_3(0),    -- Negedge data output
	C0 => CLK_IN0G,    
	C1 => CLK_IN180G,   
	CE => '1',    -- Clock enable input
	D => LEFT_CONNECTOR(32),      -- Data input (connect directly to top-level port)
	R => '0',      -- Synchronous reset input
	S => '0'       -- Synchronous preset input
);


-- De-interlace the DDR format
-- Convert from unsigned to 2's complement
REALIGN_INPUT_001: process(CLK_IN0G, DATA_IN_1, DATA_IN_3)
begin
	if rising_edge(CLK_IN0G) then
		-- realign samples 0/1
		DATA_IN_0_D(11) <= not DATA_IN_0(11);  -- I0
		DATA_IN_0_D(10 downto 0) <= DATA_IN_0(10 downto 0);  -- I0
		DATA_IN_1_D(11) <= not DATA_IN_3(11);	-- I1  
		DATA_IN_1_D(10 downto 0) <= DATA_IN_3(10 downto 0);	-- I1  
		DATA_IN_2_D(11) <= not DATA_IN_1(11);  -- Q0
		DATA_IN_2_D(10 downto 0) <= DATA_IN_1(10 downto 0);  -- Q0
		DATA_IN_3_D(11) <= not DATA_IN_2(11);	-- Q-1  
		DATA_IN_3_D(10 downto 0) <= DATA_IN_2(10 downto 0);	-- Q-1  
	end if;
end process;


--// AGC10B -----------------------------------------------------
--// Front-end AGC 
-- In order for the AGC not to track the modulation, subsampling is performed under user control.
-- The AGC loop bandwidth is controlled by the AGC_DECIMATION factor. 
-- Continuously working. 
-- Gain control is updated once every 2^(AGC_UPDATE_RATE) samples.
AGC_UPDATE_RATE <= REG0(4 downto 0);
		-- Valid range 0 - 20

AGC_001: process(AGC_UPDATE_RATE)
begin
	case(AGC_UPDATE_RATE) is
		when "00000" => AGC_DECIMATION <= x"000008"; 	-- 1x samples
		when "00001" => AGC_DECIMATION <= x"000010"; 	-- 2x samples
		when "00010" => AGC_DECIMATION <= x"000020"; 	-- 4x samples
		when "00011" => AGC_DECIMATION <= x"000040"; 	-- 8x samples
		when "00100" => AGC_DECIMATION <= x"000080"; 	-- 16x samples
		when "00101" => AGC_DECIMATION <= x"000100"; 	-- 32x samples
		when "00110" => AGC_DECIMATION <= x"000200"; 	-- 64x samples
		when "00111" => AGC_DECIMATION <= x"000400"; 	-- 128x samples
		when "01000" => AGC_DECIMATION <= x"000800"; 	-- 256x samples
		when "01001" => AGC_DECIMATION <= x"001000"; 	-- 512x samples
		when "01010" => AGC_DECIMATION <= x"002000"; 	-- 1024x samples
		when "01011" => AGC_DECIMATION <= x"004000"; 	-- 2048x samples
		when "01100" => AGC_DECIMATION <= x"008000"; 	-- 4096x samples
		when "01101" => AGC_DECIMATION <= x"010000"; 	-- 8Kx samples
		when "01110" => AGC_DECIMATION <= x"020000"; 	-- 16Kx samples
		when "01111" => AGC_DECIMATION <= x"040000"; 	-- 32Kx samples
		when "10000" => AGC_DECIMATION <= x"080000"; 	-- 64Kx samples
		when "10001" => AGC_DECIMATION <= x"100000"; 	-- 128Kx samples
		when "10010" => AGC_DECIMATION <= x"200000"; 	-- 256Kx samples
		when "10011" => AGC_DECIMATION <= x"400000"; 	-- 512Kx samples
--		when "10100" => AGC_DECIMATION <= x"800000"; 	-- 1Mx samples
		when others => AGC_DECIMATION <= x"800000"; 	-- 1Mx samples
	end case;
end process;

AGC_002: AGC10B port map(
	CLK => CLK_IN0G,
	DATA_I_IN => DATA_IN_0_D,  -- AGC10 expects a complex input. Fake it.
	DATA_Q_IN => DATA_IN_2_D,
	SAMPLE_CLK_IN => '1',
	DECIMATION => AGC_DECIMATION,
	TARGET_MAGNITUDE => AGC_TARGET_MAGNITUDE,
	AGC_DAC => AGC_DAC,
	AGC_DAC_SAMPLE_CLK => AGC_DAC_SAMPLE_CLK,
	S2 => open,  
	S2_CLK => open
	);


		
--// COMSCOPE -----------------------------------------------------
-- Define Comscope traces here 
-- Trace 1
S1_1 <= DATA_IN_0_D(11 downto 4);  -- I0
S1_1_CLK <= '1';
S1_2 <= DATA_IN_1_D(11 downto 4);  -- I1
S1_2_CLK <= '1';
S1_3 <= DATA_IN_2_D(11 downto 4);  -- Q0
S1_3_CLK <= '1';
S1_4 <= DATA_IN_3_D(11 downto 4);  -- Q-1
S1_4_CLK <= '1';

-- Trace 2
S2_1 <= DATA_IN_1_D(11 downto 4);
S2_1_CLK <= '1';
S2_2 <= AGC_DAC(9 downto 2);
S2_2_CLK <= AGC_DAC_SAMPLE_CLK;
S2_3 <= DATA_IN_3_D(11 downto 4);
S2_3_CLK <= '1';
S2_4 <=  DATA_IN_2_D(11 downto 4);  
S2_4_CLK <= '1';

-- Trigger (1-bit format, undefined)
T1_CLK <= '1';
T1 <= '1';	-- start of PRBS11 test sequence

S1_1B_CLK <= S1_1_CLK or REG240(7);	-- Nominal sampling clock or real-time processing clock
S1_2B_CLK <= S1_2_CLK or REG240(7);	-- Nominal sampling clock or real-time processing clock
S1_3B_CLK <= S1_3_CLK or REG240(7);	-- Nominal sampling clock or real-time processing clock
S1_4B_CLK <= S1_4_CLK or REG240(7);	-- Nominal sampling clock or real-time processing clock

S2_1B_CLK <= S2_1_CLK or REG242(7);	-- Nominal sampling clock or real-time processing clock
S2_2B_CLK <= S2_2_CLK or REG242(7);	-- Nominal sampling clock or real-time processing clock
S2_3B_CLK <= S2_3_CLK or REG242(7);	-- Nominal sampling clock or real-time processing clock
S2_4B_CLK <= S2_4_CLK or REG242(7);	-- Nominal sampling clock or real-time processing clock

COMSCOPE_001: COMSCOPE port map(
   CLK => CLK_P,
	CLK_IN => CLK_IN0G,
	ASYNC_RESET => ASYNC_RESET,
	SIGNED_REPRESENTATION_U_SIGNED_N => REG248(6),	-- trigger format signed(1) or unsigned(0)
	REG237 => REG237,
	REG238 => REG238,
	REG239 => REG239,
	REG240 => REG240,
	REG241 => REG241,
	REG242 => REG242,
	REG243 => REG243,
	REG244 => REG244,
	REG245 => REG245,
	REG246 => REG246,
	REG247 => REG247,
	REG248 => REG248,
	REG249 => REG249,
	SIGNAL_1_BIT_1_1 => '0',
	SIGNAL_1_BIT_SAMPLE_CLK_1_1 => '0',
	SIGNAL_1_BIT_1_2 => '0',
	SIGNAL_1_BIT_SAMPLE_CLK_1_2 => '0',
	SIGNAL_1_BIT_1_3 => '0',
	SIGNAL_1_BIT_SAMPLE_CLK_1_3 => '0',
	SIGNAL_1_BIT_1_4 => '0',
	SIGNAL_1_BIT_SAMPLE_CLK_1_4 => '0',
	SIGNAL_2_BIT_1_1 => (others => '0'),
	SIGNAL_2_BIT_SAMPLE_CLK_1_1 => '0',
	SIGNAL_2_BIT_1_2 => (others => '0'),
	SIGNAL_2_BIT_SAMPLE_CLK_1_2 => '0',
	SIGNAL_2_BIT_1_3 => (others => '0'),
	SIGNAL_2_BIT_SAMPLE_CLK_1_3 => '0',
	SIGNAL_2_BIT_1_4 => (others => '0'),
	SIGNAL_2_BIT_SAMPLE_CLK_1_4 => '0',
	SIGNAL_4_BIT_1_1 => (others => '0'),
	SIGNAL_4_BIT_SAMPLE_CLK_1_1 => '0',
	SIGNAL_4_BIT_1_2 => (others => '0'),
	SIGNAL_4_BIT_SAMPLE_CLK_1_2 => '0',
	SIGNAL_4_BIT_1_3 => (others => '0'),
	SIGNAL_4_BIT_SAMPLE_CLK_1_3 => '0',
	SIGNAL_4_BIT_1_4 => (others => '0'),
	SIGNAL_4_BIT_SAMPLE_CLK_1_4 => '0',
	SIGNAL_8_BIT_1_1 => S1_1,
	SIGNAL_8_BIT_SAMPLE_CLK_1_1 => S1_1B_CLK,
	SIGNAL_8_BIT_1_2 => S1_2,
	SIGNAL_8_BIT_SAMPLE_CLK_1_2 => S1_2B_CLK,
	SIGNAL_8_BIT_1_3 => S1_3,
	SIGNAL_8_BIT_SAMPLE_CLK_1_3 => S1_3B_CLK,
	SIGNAL_8_BIT_1_4 => S1_4,
	SIGNAL_8_BIT_SAMPLE_CLK_1_4 => S1_4B_CLK,
	SIGNAL_16_BIT_1_1 => (others => '0'),
	SIGNAL_16_BIT_SAMPLE_CLK_1_1 => '0',
	SIGNAL_16_BIT_1_2 => (others => '0'),
	SIGNAL_16_BIT_SAMPLE_CLK_1_2 => '0',
	SIGNAL_16_BIT_1_3 => (others => '0'),
	SIGNAL_16_BIT_SAMPLE_CLK_1_3 => '0',
	SIGNAL_16_BIT_1_4 => (others => '0'),
	SIGNAL_16_BIT_SAMPLE_CLK_1_4 => '0',
	SIGNAL_1_BIT_2_1 => '0',
	SIGNAL_1_BIT_SAMPLE_CLK_2_1 => '0',
	SIGNAL_1_BIT_2_2 => '0',
	SIGNAL_1_BIT_SAMPLE_CLK_2_2 => '0',
	SIGNAL_1_BIT_2_3 => '0',
	SIGNAL_1_BIT_SAMPLE_CLK_2_3 => '0',
	SIGNAL_1_BIT_2_4 => '0',
	SIGNAL_1_BIT_SAMPLE_CLK_2_4 => '0',
	SIGNAL_2_BIT_2_1 => (others => '0'),
	SIGNAL_2_BIT_SAMPLE_CLK_2_1 => '0',
	SIGNAL_2_BIT_2_2 => (others => '0'),
	SIGNAL_2_BIT_SAMPLE_CLK_2_2 => '0',
	SIGNAL_2_BIT_2_3 => (others => '0'),
	SIGNAL_2_BIT_SAMPLE_CLK_2_3 => '0',
	SIGNAL_2_BIT_2_4 => (others => '0'),
	SIGNAL_2_BIT_SAMPLE_CLK_2_4 => '0',
	SIGNAL_4_BIT_2_1 => (others => '0'),
	SIGNAL_4_BIT_SAMPLE_CLK_2_1 => '0',
	SIGNAL_4_BIT_2_2 => (others => '0'),
	SIGNAL_4_BIT_SAMPLE_CLK_2_2 => '0',
	SIGNAL_4_BIT_2_3 => (others => '0'),
	SIGNAL_4_BIT_SAMPLE_CLK_2_3 => '0',
	SIGNAL_4_BIT_2_4 => (others => '0'),
	SIGNAL_4_BIT_SAMPLE_CLK_2_4 => '0',
	SIGNAL_8_BIT_2_1 => S2_1,
	SIGNAL_8_BIT_SAMPLE_CLK_2_1 => S2_1B_CLK,
	SIGNAL_8_BIT_2_2 => S2_2,
	SIGNAL_8_BIT_SAMPLE_CLK_2_2 => S2_2B_CLK,
	SIGNAL_8_BIT_2_3 => S2_3,
	SIGNAL_8_BIT_SAMPLE_CLK_2_3 => S2_3B_CLK,
	SIGNAL_8_BIT_2_4 => S2_4,
	SIGNAL_8_BIT_SAMPLE_CLK_2_4 => S2_4B_CLK,
	SIGNAL_16_BIT_2_1 => (others => '0'),
	SIGNAL_16_BIT_SAMPLE_CLK_2_1 => '0',
	SIGNAL_16_BIT_2_2 => (others => '0'),
	SIGNAL_16_BIT_SAMPLE_CLK_2_2 => '0',
	SIGNAL_16_BIT_2_3 => (others => '0'),
	SIGNAL_16_BIT_SAMPLE_CLK_2_3 => '0',
	SIGNAL_16_BIT_2_4 => (others => '0'),
	SIGNAL_16_BIT_SAMPLE_CLK_2_4 => '0',
	SIGNAL_1_BIT_3_1 => '0',
	SIGNAL_1_BIT_SAMPLE_CLK_3_1 => '0',
	SIGNAL_1_BIT_3_2 => '0',
	SIGNAL_1_BIT_SAMPLE_CLK_3_2 => '0',
	SIGNAL_1_BIT_3_3 => '0',
	SIGNAL_1_BIT_SAMPLE_CLK_3_3 => '0',
	SIGNAL_1_BIT_3_4 => '0',
	SIGNAL_1_BIT_SAMPLE_CLK_3_4 => '0',
	SIGNAL_2_BIT_3_1 => (others => '0'),
	SIGNAL_2_BIT_SAMPLE_CLK_3_1 => '0',
	SIGNAL_2_BIT_3_2 => (others => '0'),
	SIGNAL_2_BIT_SAMPLE_CLK_3_2 => '0',
	SIGNAL_2_BIT_3_3 => (others => '0'),
	SIGNAL_2_BIT_SAMPLE_CLK_3_3 => '0',
	SIGNAL_2_BIT_3_4 => (others => '0'),
	SIGNAL_2_BIT_SAMPLE_CLK_3_4 => '0',
	SIGNAL_4_BIT_3_1 => (others => '0'),
	SIGNAL_4_BIT_SAMPLE_CLK_3_1 => '0',
	SIGNAL_4_BIT_3_2 => (others => '0'),
	SIGNAL_4_BIT_SAMPLE_CLK_3_2 => '0',
	SIGNAL_4_BIT_3_3 => (others => '0'),
	SIGNAL_4_BIT_SAMPLE_CLK_3_3 => '0',
	SIGNAL_4_BIT_3_4 => (others => '0'),
	SIGNAL_4_BIT_SAMPLE_CLK_3_4 => '0',
	SIGNAL_8_BIT_3_1 => (others => '0'),
	SIGNAL_8_BIT_SAMPLE_CLK_3_1 => '0',
	SIGNAL_8_BIT_3_2 => (others => '0'),
	SIGNAL_8_BIT_SAMPLE_CLK_3_2 => '0',
	SIGNAL_8_BIT_3_3 => (others => '0'),
	SIGNAL_8_BIT_SAMPLE_CLK_3_3 => '0',
	SIGNAL_8_BIT_3_4 => (others => '0'),
	SIGNAL_8_BIT_SAMPLE_CLK_3_4 => '0',
	SIGNAL_16_BIT_3_1 => (others => '0'),
	SIGNAL_16_BIT_SAMPLE_CLK_3_1 => '0',
	SIGNAL_16_BIT_3_2 => (others => '0'),
	SIGNAL_16_BIT_SAMPLE_CLK_3_2 => '0',
	SIGNAL_16_BIT_3_3 => (others => '0'),
	SIGNAL_16_BIT_SAMPLE_CLK_3_3 => '0',
	SIGNAL_16_BIT_3_4 => (others => '0'),
	SIGNAL_16_BIT_SAMPLE_CLK_3_4 => '0',
	SIGNAL_1_BIT_4_1 => '0',
	SIGNAL_1_BIT_SAMPLE_CLK_4_1 => '0',
	SIGNAL_1_BIT_4_2 => '0',
	SIGNAL_1_BIT_SAMPLE_CLK_4_2 => '0',
	SIGNAL_1_BIT_4_3 => '0',
	SIGNAL_1_BIT_SAMPLE_CLK_4_3 => '0',
	SIGNAL_1_BIT_4_4 => '0',
	SIGNAL_1_BIT_SAMPLE_CLK_4_4 => '0',
	SIGNAL_2_BIT_4_1 => (others => '0'),
	SIGNAL_2_BIT_SAMPLE_CLK_4_1 => '0',
	SIGNAL_2_BIT_4_2 => (others => '0'),
	SIGNAL_2_BIT_SAMPLE_CLK_4_2 => '0',
	SIGNAL_2_BIT_4_3 => (others => '0'),
	SIGNAL_2_BIT_SAMPLE_CLK_4_3 => '0',
	SIGNAL_2_BIT_4_4 => (others => '0'),
	SIGNAL_2_BIT_SAMPLE_CLK_4_4 => '0',
	SIGNAL_4_BIT_4_1 => (others => '0'),
	SIGNAL_4_BIT_SAMPLE_CLK_4_1 => '0',
	SIGNAL_4_BIT_4_2 => (others => '0'),
	SIGNAL_4_BIT_SAMPLE_CLK_4_2 => '0',
	SIGNAL_4_BIT_4_3 => (others => '0'),
	SIGNAL_4_BIT_SAMPLE_CLK_4_3 => '0',
	SIGNAL_4_BIT_4_4 => (others => '0'),
	SIGNAL_4_BIT_SAMPLE_CLK_4_4 => '0',
	SIGNAL_8_BIT_4_1 => (others => '0'),
	SIGNAL_8_BIT_SAMPLE_CLK_4_1 => '0',
	SIGNAL_8_BIT_4_2 => (others => '0'),
	SIGNAL_8_BIT_SAMPLE_CLK_4_2 => '0',
	SIGNAL_8_BIT_4_3 => (others => '0'),
	SIGNAL_8_BIT_SAMPLE_CLK_4_3 => '0',
	SIGNAL_8_BIT_4_4 => (others => '0'),
	SIGNAL_8_BIT_SAMPLE_CLK_4_4 => '0',
	SIGNAL_16_BIT_4_1 => (others => '0'),
	SIGNAL_16_BIT_SAMPLE_CLK_4_1 => '0',
	SIGNAL_16_BIT_4_2 => (others => '0'),
	SIGNAL_16_BIT_SAMPLE_CLK_4_2 => '0',
	SIGNAL_16_BIT_4_3 => (others => '0'),
	SIGNAL_16_BIT_SAMPLE_CLK_4_3 => '0',
	SIGNAL_16_BIT_4_4 => (others => '0'),
	SIGNAL_16_BIT_SAMPLE_CLK_4_4 => '0',
	TRIGGER_1_BIT_1 => T1,
	TRIGGER_1_BIT_SAMPLE_CLK_1 => T1_CLK,
	TRIGGER_1_BIT_2 => T2,
	TRIGGER_1_BIT_SAMPLE_CLK_2 => T2_CLK,
	TRIGGER_1_BIT_3 => '0',
	TRIGGER_1_BIT_SAMPLE_CLK_3 => '0',
	TRIGGER_1_BIT_4 => '0',
	TRIGGER_1_BIT_SAMPLE_CLK_4 => '0',
	TRIGGER_2_BIT_1 => (others => '0'),
	TRIGGER_2_BIT_SAMPLE_CLK_1 => '0',
	TRIGGER_2_BIT_2 => (others => '0'),
	TRIGGER_2_BIT_SAMPLE_CLK_2 => '0',
	TRIGGER_2_BIT_3 => (others => '0'),
	TRIGGER_2_BIT_SAMPLE_CLK_3 => '0',
	TRIGGER_2_BIT_4 => (others => '0'),
	TRIGGER_2_BIT_SAMPLE_CLK_4 => '0',
	TRIGGER_4_BIT_1 => (others => '0'),
	TRIGGER_4_BIT_SAMPLE_CLK_1 => '0',
	TRIGGER_4_BIT_2 => (others => '0'),
	TRIGGER_4_BIT_SAMPLE_CLK_2 => '0',
	TRIGGER_4_BIT_3 => (others => '0'),
	TRIGGER_4_BIT_SAMPLE_CLK_3 => '0',
	TRIGGER_4_BIT_4 => (others => '0'),
	TRIGGER_4_BIT_SAMPLE_CLK_4 => '0',
	TRIGGER_8_BIT_1 => (others => '0'),
	TRIGGER_8_BIT_SAMPLE_CLK_1 => '0',
	TRIGGER_8_BIT_2 => (others => '0'),
	TRIGGER_8_BIT_SAMPLE_CLK_2 => '0',
	TRIGGER_8_BIT_3 => (others => '0'),
	TRIGGER_8_BIT_SAMPLE_CLK_3 => '0',
	TRIGGER_8_BIT_4 => (others => '0'),
	TRIGGER_8_BIT_SAMPLE_CLK_4 => '0',
	TRIGGER_16_BIT_1 => (others => '0'),
	TRIGGER_16_BIT_SAMPLE_CLK_1 => '0',
	TRIGGER_16_BIT_2 => (others => '0'),
	TRIGGER_16_BIT_SAMPLE_CLK_2 => '0',
	TRIGGER_16_BIT_3 => (others => '0'),
	TRIGGER_16_BIT_SAMPLE_CLK_3 => '0',
	TRIGGER_16_BIT_4 => (others => '0'),
	TRIGGER_16_BIT_SAMPLE_CLK_4 => '0',
	REG250 => REG250,
	REG251 => REG251,
	TRIGGER_REARM_TOGGLE => TRIGGER_REARM_TOGGLE,
	FORCE_TRIGGER_TOGGLE => FORCE_TRIGGER_TOGGLE,
	REG250_READ => REG250_READ,
	START_CAPTURE_TOGGLE => START_CAPTURE_TOGGLE
);

DUMMY_OUT <= SERIAL_COM_2 or SERIAL_COM_1;
-- prevents pullup on these signals		


--// TEST POINTS -------------
-- test signal generator
TEST_POINTS(1) <= CLK_IN0G;
TEST_POINTS(2) <= CLK_IN180G;
TEST_POINTS(3) <= DCM_LOCKED_001;
TEST_POINTS(4) <= DCM_LOCKED_003;
TEST_POINTS(5) <= CLK_P;
TEST_POINTS(6) <= '0';
TEST_POINTS(7) <= '0';
TEST_POINTS(8) <= '0';
TEST_POINTS(9) <= '0';
TEST_POINTS(10) <= '0';

 
--// MONITORING & CONTROL -------------
--// Serial interfaces are used for module-to-module communication.
-- FOR MOST APPLICATIONS, KEEP THIS SECTION (UNTIL THE END) UNCHANGED.
-- REG255 reserved for reading 4 serial port inputs
-- These signals are pulled high (to mimic stop bit).

REG255(0) <= RX_L	;	-- port 1 left = ComBlock 40-pin interface
REG255(1) <= RX_B	;	-- port 2 bottom = Comblock 40-pin interface to external transceiver
REG255(2) <= RX_R ;	
REG255(3) <= RX_USB_BUFFER_EMPTY; 	
	-- When low, uC must read byte from USB or LAN. 
REG255(7 downto 4) <= "0000";

-- Reclock TX_USB_TOGGLE
RECLOCK_TX_USB_TOGGLE: process(CLK_P,TX_USB_TOGGLE)
begin
	if rising_edge(CLK_P) then
		TX_USB_TOGGLE_D <= TX_USB_TOGGLE;
		TX_USB_TOGGLE_D2 <= TX_USB_TOGGLE_D;
		if(TX_USB_TOGGLE_D /= TX_USB_TOGGLE_D2) then
			TX_USB_SAMPLE_CLK <= '1';
		else
			TX_USB_SAMPLE_CLK <= '0';
		end if;
	end if;
end process;


--// Atmel microcontroller interface
-- Used for monitoring and control.
-- address latch
UC_ADDR_001: process(UC_ALE, UC_FLASH_CSN, UC_AD)
begin
	if falling_edge(UC_ALE) then
		if(UC_FLASH_CSN = '1') then
			UC_ADDRESS <= UC_AD;
	  	end if;
  	end if;
end process;

RECLOCK_ADDR_D: process(ASYNC_RESET, CLK_P, UC_ADDRESS)
begin
	if(ASYNC_RESET = '1') then
		UC_ADDR_D <= (others =>'0');
		UC_ADDR_D2 <= (others =>'0');
	elsif rising_edge(CLK_P) then
		UC_ADDR_D <= UC_ADDRESS;
		UC_ADDR_D2 <= UC_ADDR_D;
	end if;
end process;

-- Immediately upon completing reading a byte from the USB or LAN, ask for the 
-- next one. 
RECLOCK_RDN_D: process(CLK_P, UC_RDN)
begin
	if rising_edge(CLK_P) then
		UC_RDN_D <= UC_RDN;
		UC_RDN_D2 <= UC_RDN_D;
		RX_USB_BUFFER_EMPTY_D <= RX_USB_BUFFER_EMPTY;
		RX_USB_BUFFER_EMPTY_D2 <= RX_USB_BUFFER_EMPTY_D;

		if (UC_RDN_D = '1') and (UC_RDN_D2 = '0') and (UC_ADDR_D2 = 254)then
			-- end of M&C byte read from Atmel uC.
			if(RX_USB_BUFFER_EMPTY_D2 = '0') then
				-- last M&C byte read was over USB. Increment USB elastic buffer read pointer
				RX_USB_SAMPLE_CLK_REQ <= '1';
			end if;
		else
			RX_USB_SAMPLE_CLK_REQ <= '0';
		end if;
	end if;
end process;

-- read
UC_READ_001: process(UC_FLASH_CSN, UC_RDN, UC_ADDRESS, SREG0,  
	REG250, REG251, REG255, RX_USB, RX_USB_BUFFER_EMPTY)
begin
	if(UC_FLASH_CSN = '1') and (UC_RDN = '0') then
		if(UC_ADDRESS(7 downto 2) = "111111") then
			if(UC_ADDRESS(1 downto 0) = "11") then
				UC_AD <= REG255;			-- reg 255. Async serial from 3 connectors
			elsif(UC_ADDRESS(1 downto 0) = "10") then
				-- byte received over USB
				UC_AD <= RX_USB;		-- reg 254 = 8-bit data from USB M&C channel
			elsif(UC_ADDRESS(1 downto 0) = "01") then
				UC_AD <= REVISION;	-- reg 253 = fpga revision
			elsif(UC_ADDRESS(1 downto 0) = "00") then
				UC_AD <= OPTION;		-- reg 252 = fpga option
			else
				UC_AD <= "ZZZZZZZZ";
			end if;
		elsif(UC_ADDRESS= 0) then
			UC_AD <= SREG0;
		elsif (UC_ADDRESS = 251) then
			UC_AD <= REG251;		-- reg 251 =comscope  status
		elsif (UC_ADDRESS = 250) then
			UC_AD <= REG250;		-- reg 250 = comscope data byte
		else
			UC_AD <= "ZZZZZZZZ";
		end if;
	else
		UC_AD <= "ZZZZZZZZ";
 	end if;
end process;

-- detect when address 250 is being read
REG250_READ <= '1' when (UC_FLASH_CSN = '1') and (UC_RDN = '0') and (UC_ADDRESS = 250) else '0';

-- write. 
-- Registers 0 through 15 are control registers.
UC_WRITE_001: process(ASYNC_RESET, UC_FLASH_CSN, UC_WRN, UC_ADDRESS, UC_AD)
begin
	if(ASYNC_RESET = '1') then
		-- CAREFUL!!!!!! 
		-- when the external clock comes from a bigger FPGA, the ASYNC_RESET
		-- may arrive here AFTER the Atmel uC has configured the registers.
		-- We should avoid resetting the registers.
		TX_L <= '1';	-- stop bit
		TX_B <= '1';
		TX_R <= '1';
		-- registers initialization for simulation
		REG0 <= x"0A"; -- AGC averaging 1024 bits, NRZ
	 elsif falling_edge(UC_WRN) then

		if (UC_ADDRESS(7 downto 4) = 0) then
			if (UC_ADDRESS(3 downto 0) = 0) then
				REG0 <= UC_AD;
 				CONFIG_CHANGE_TOGGLE <= not CONFIG_CHANGE_TOGGLE;	
				-- Configuration change toggles upon writing the last control register
				-- This happens invisibly to the user (only the Atmel uC writes to the last register.
				-- which is invible to the user and to the ComBlock Control Center).
			end if;
		end if;
		if (UC_ADDRESS = 254) then
			-- REG254 reserved for output serial
			TX_L <= UC_AD(0);
			TX_B <= UC_AD(1);
			-- right connector serial output pin depends on whether 
			-- a LAN adapter is used or not on the same connector.
			-- Be sure to disable TX_R pin at the output mux.
			TX_R <= UC_AD(2);
		end if;
		if (UC_ADDRESS = 255) then
			-- REG255 write reserved for output serial (8-bit parallel format)
			TX_USB <= UC_AD;
			TX_USB_TOGGLE <= not TX_USB_TOGGLE;
		end if;
	end if;
end process;

--// reserved COMSCOPE
UC_WRITE_002: process(UC_FLASH_CSN, UC_WRN, UC_ADDRESS, UC_AD, 
	FORCE_TRIGGER_TOGGLE, TRIGGER_REARM_TOGGLE, START_CAPTURE_TOGGLE)
begin
	if falling_edge(UC_WRN) then
		if(UC_FLASH_CSN = '1') then
			-- Reserved for ComScope			
			if (UC_ADDRESS = 237) then
				REG237 <= UC_AD;	
			end if;
			if (UC_ADDRESS = 238) then
				REG238 <= UC_AD;	
			end if;
			if (UC_ADDRESS = 239) then
				REG239 <= UC_AD;
			end if;
			if (UC_ADDRESS = 240) then
				REG240 <= UC_AD;	
			end if;
			if (UC_ADDRESS = 241) then
				REG241 <= UC_AD;	
			end if;
			if (UC_ADDRESS = 242) then
				REG242 <= UC_AD;	
			end if;
			if (UC_ADDRESS = 243) then
				REG243 <= UC_AD;	
			end if;
			if (UC_ADDRESS = 244) then
				REG244 <= UC_AD;	
			end if;
			if (UC_ADDRESS = 245) then
				REG245 <= UC_AD;	
			end if;
			if (UC_ADDRESS = 246) then
				REG246 <= UC_AD;	
			end if;
			if (UC_ADDRESS = 247) then
				REG247 <= UC_AD;	
			end if;
			if (UC_ADDRESS = 248) then
				REG248 <= UC_AD;	
			end if;
			if (UC_ADDRESS = 249) then
				REG249 <= UC_AD;	
			end if;
			if (UC_ADDRESS = 250) then 
				-- detect one-shot force trigger	
				if(UC_AD(0) = '1') then	
					FORCE_TRIGGER_TOGGLE <= not(FORCE_TRIGGER_TOGGLE);
				end if;	
				-- detect one-shot re-arm	
				if(UC_AD(1) = '1') then	
					TRIGGER_REARM_TOGGLE <= not(TRIGGER_REARM_TOGGLE);
				end if;	
				-- start capturing again
				if(UC_AD(2) = '1') then	
					START_CAPTURE_TOGGLE <= not (START_CAPTURE_TOGGLE);
				end if;	
			end if;
		end if;
	end if;
end process;


end Behavioral;
